# -*- coding: utf-8 -*-

import mimetools, mimetypes, smtplib
import os
from cStringIO import StringIO

from email import Header
from email.Charset import *
from email import Encoders
from email.Message import Message
from email.MIMEAudio import MIMEAudio
from email.MIMEBase import MIMEBase
from email.MIMEMultipart import MIMEMultipart
from email.MIMEImage import MIMEImage
from email.MIMEText import MIMEText
import email.Utils

from tempfile import NamedTemporaryFile
import commands, os

from mimetypes import guess_type

class MyObject:
    pass

if not __builtins__.has_key("object"):
    __builtins__["object"] = MyObject

class MailWithAttachment(object):
    def __init__(self, mfrom, charset, smtp = None, lynx_command = "lynx", lynx_options = ""):
        self._msg = MIMEMultipart()
        self._msg['From'] = mfrom
        self._smtp = smtp
        self._charset = charset
        self._id = email.Utils.make_msgid()
        self._lynx_command = lynx_command
        self._lynx_options = lynx_options
        self._msg['Message-ID'] = self._id

    def setId(self, _id):
        """
        Set id match string
        """
        self._id = email.Utils.make_msgid(_id)
        self._msg['Message-ID'] = self._id

    def setFrom(self, _from):
        """
        Set the sender address
        """
        self._msg['From'] = _from

    def getFrom(self):
        """
        Return the sender address, or None if not defined
        """
        return self._msg['From']

    def setSmtp(self, _stmp):
        """
        Set the SMTP Server
        """
        self._smtp = _smtp

    def setTo(self, to):
        """
        Set the recipient address
        """
        self._msg['To'] = to

    def getTo(self):
        """
        Return the recipient address, or None if not defined
        """
        if 'To' in self._msg:
            return self._msg['To']
        return None

    def setSubject(self, subject):
        """
        Set the mail subject
        """
        if isinstance(subject, unicode):
            subject = subject.encode(self._charset)
        self._msg['Subject'] = str(Header.Header(subject, self._charset))
        self._msg.preambule = self._msg['Subject']
        # Guarantees the message ends in a newline
        self._msg.epilogue = ''

    def getSubject(self):
        """
        Return the mail subject
        """
        return self._msg['Subject']

    def setReply(self, reply):
        """
        Set the reply-to field
        """
        self._msg['Reply-to'] = reply

    def setBody(self, body):
        """
        Try to encode the body text and add it to the message
        """
        self._text = MIMEText(body, _subtype='plain', _charset=self._charset)
        self._msg.attach(self._text)

    def convert_html(self, body):
        """
        Convert HTML to text using Lynx
        """
        f = NamedTemporaryFile(suffix='.html')
        f.write(body)
        f.flush()
        cmd = '%s -dump -nolist %s file://%s' % (self._lynx_command,
                                                 self._lynx_options, f.name)
        result, plaintext = commands.getstatusoutput(cmd)
        f.close()
        if result != 0:
            return ""
        return plaintext

    def setHtmlBody(self, body, text = None):
        """
        Set an HTML body.
        If text is set to None or False, only HTML will be included.
        If text is a string, it'll be the text alternative.
        If text is True, Mailnode will convert the HTML to plain text
        using Lynx.
        """
        html = MIMEText(body, _subtype='html', _charset=self._charset)
        if text:
            if not isinstance(text, str):
                text = self.convert_html(body)
            text = MIMEText(text, _subtype='plain', _charset=self._charset)
            part = MIMEMultipart("alternative")
            part.attach(text)
            part.attach(html)
        else:
            part = html
        self._text = part
        self._msg.attach(self._text)

    def addAttachment(self, path='',**kwargs):
        """
        Add an attachment, trying to guess the mime type
        """
        if kwargs.has_key('data'):
            data = str(kwargs.get('data'))
            fileName = kwargs.get('filename','attach.bin')
            content_type, encoding=mimetypes.guess_type(fileName)
            ctype = kwargs.get('mimetype',content_type)
            encoding = None
        else:
            if not os.path.isfile(path):
                raise IOError, 'InvalidPath %s' % path
            ctype, encoding = mimetypes.guess_type(path)
            fp = open(path)
            data = fp.read()
            fp.close()
            fileName = path.split('/')[-1]
            fileName = fileName.split('\\')[-1]
            fileName = fileName.split(':')[-1]
        if ctype is None or encoding is not None:
            ctype = 'application/octet-stream'
        maintype, subtype = ctype.split('/', 1)
        if maintype == 'text':
            # Note: we should handle calculating the charset
            attach = MIMEText(data, _subtype=subtype, _charset=self._charset)
        elif maintype == 'image':
            attach = MIMEImage(data, _subtype=subtype)
        elif maintype == 'audio':
            attach = MIMEAudio(data, _subtype=subtype)
        else:
            attach = MIMEBase(maintype, subtype)
            attach.set_payload(data)
            # Encode the payload using Base64
            Encoders.encode_base64(attach)
        # Set the filename parameter
        self._filename = fileName
        attach.add_header('Content-Disposition', 'attachment', filename=fileName)
        self._msg.attach(attach)

    def getId(self):
        """
        Return the message id
        """
        return self._id
    
    def as_string(self):
        """
        Get full message as string
        """
        return self._msg.as_string()

    def sendMail(self):
        """
        Send the mail via SMTP
        """
        if self._msg['To']:
            s = smtplib.SMTP(self._smtp)
            s.sendmail(self._msg['From'], self._msg['To'], self._msg.as_string())
            s.close()
        else:
            raise RuntimeError, 'Mailnode: NoRecipientDefined'

