# -*- coding: utf-8 -*-

from OFS.PropertyManager import PropertyManager
from OFS.SimpleItem import SimpleItem
from AccessControl import ClassSecurityInfo
from MailWithAttachment import MailWithAttachment
from Globals import InitializeClass, DTMLFile

from random import choice
import poplib, socket
import re, string

import zLOG

VIEW_PERMISSION = 'View'


manage_addMailnodeForm=DTMLFile('dtml/addMailnode_form', globals())


def manage_addMailnode( self, id, REQUEST=None ):
    """
    Add a Mailnode into the system
    """

    obj = Mailnode(id)
    self._setObject(id, obj)

    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')

add = manage_addMailnode



class Mailnode(PropertyManager, SimpleItem):
    """
    This tool encapsulates mail send and verification.
    """

    meta_type = "Mailnode"
    id = "Mailnode"
    title = "Mailnode mail sender"
    icon_file = 'mailnode.gif'    
    
    security = ClassSecurityInfo()

    manage_options = PropertyManager.manage_options + SimpleItem.manage_options


    _properties = (
        {'id': 'from_address', 'type': 'string', 'mode': 'rw', "label":"Default From address"},
        {'id': 'charset', 'type': 'string', 'mode': 'rw', "label":"Default charset"},
        {'id': 'message_id_catcher', 'type': 'string', 'mode': 'rw', "label":"Message id catcher"},
        {'id': 'pop_server', 'type': 'string', 'mode': 'rw', "label":"POP server for error checkin"},
        {'id': 'pop_port', 'type': 'int', 'mode': 'rw', "label":"POP server port"},
        {'id': 'pop_username', 'type': 'string', 'mode': 'rw', "label":"POP server login"},
        {'id': 'pop_password', 'type': 'string', 'mode': 'rw', "label":"POP server password"},
        {'id': 'error_callback', 'type': 'string', 'mode': 'rw', "label":"Callback in case of error"},
        {'id': 'lynx_command', 'type': 'string', 'mode': 'rw', "label":"Lynx binary for HTML conversion"},
        {'id': 'lynx_options', 'type': 'string', 'mode': 'rw', "label":"Lynx options"},
        )

    from_address = ""
    charset = ""
    pop_server = ""
    pop_port = 110
    pop_username = ""
    pop_password = ""
    error_callback = ""
    lynx_command = "lynx"
    lynx_options = ""
    message_id_catcher = ""

    def __init__(self, *args, **kwargs):
        """ Init tool properties """

        letters = ''.join([ x for x in string.letters if ord(x) < 128 ])
        chars = letters + string.digits
        msg_id = ''.join([ choice(chars) for i in range(8) ])

        self.sessions = {}

        self.message_id_catcher = msg_id

    security.declareProtected(VIEW_PERMISSION, 'getFrom_address')
    def getFrom_address(self):
        """
        Get from address, either from Mailnode or from portal properties
        """
        mfrom =self.getProperty("from_address", None)
        if not mfrom:
            try:
                portal = self.portal_url.getPortalObject()
                mfrom = portal.getProperty("email_from_address")
            except:
                pass
        return mfrom

    security.declareProtected(VIEW_PERMISSION, 'getCharset')
    def getCharset(self):
        """
        Get charset, either from Mailnode or from portal_properties
        """
        charset = self.getProperty("charset", '')
        default_charset = self.portal_properties.site_properties.getProperty("default_charset", 'utf-8')
        return charset or default_charset

    security.declareProtected(VIEW_PERMISSION, 'sendMail')
    def sendMail(self, arg = {}, **kwargs):
        """
        Send a mail, and return mail id
        """
        arg.update(kwargs)
        mfrom = arg.get("from", self.getFrom_address())
        mail = MailWithAttachment(mfrom,
                                  self.getCharset())
        mail.setTo(arg['to'])
        mail.setSubject(arg['subject'])
        mail.setBody(arg['body'])
        mail.setId(self.getProperty('message_id_catcher'))
        if arg.has_key('attachment'):
            for attachment in arg['attachment']:
                if type(attachment) == type({}):
                    mail.addAttachment(**attachment)
                else:
                    mail.addAttachment(attachment)
        self.send(mail)
        return mail.getId()

    security.declareProtected(VIEW_PERMISSION, 'getMailObject')
    def getMailObject(self, arg = {}, **kwargs):
        """
        Get a mail object, to append stuff
        """
        arg.update(kwargs)
        mfrom = arg.get("from", self.getFrom_address())
        mail = MailWithAttachment(mfrom,
                                  self.getCharset(),
                                  lynx_command = self.lynx_command,
                                  lynx_options = self.lynx_options)
        mail.setTo(arg['to'])
        mail.setSubject(arg['subject'])
        mail.setId(self.getProperty('message_id_catcher'))
        return mail

    security.declareProtected(VIEW_PERMISSION, 'send')
    def send(self, mail):
        """
        Send a mail object using MailHost
        """
        mailhost = self.MailHost
        mailhost.send(mail.as_string(), mail.getTo(), mail.getFrom())

    security.declareProtected(VIEW_PERMISSION, 'check')
    def check(self):
        """
        Do the pop3 checking
        """
        M = poplib.POP3(self.getProperty('pop_server'),
			self.getProperty('pop_port'))
        M.user(self.getProperty('pop_username'))
        M.pass_(self.getProperty('pop_password'))

        func = self.getParentNode().restrictedTraverse(self.getProperty('error_callback'))
        
        messages = M.list()[1]
        numBadMessages = 0
        reg = re.compile('Message-ID:.*(<.*%s@%s>).*' %
                            (self.getProperty('message_id_catcher'),
                             socket.getfqdn()))
        for i in messages:
            id, _ = i.split() 
            for j in M.retr(id)[1]:
                for line in j.splitlines():
                    err = reg.match(line)
                    if err:
                        numBadMessages += 1
                        func(err.groups()[0].strip())
            M.dele(id)
        M.quit()
        return '%d/%d' % (numBadMessages, len(messages))

InitializeClass(Mailnode)
